@@ -24,6 +24,7 @@
 #include <QEvent>
 #include <QList>
 #include <QTreeWidget>
+#include <QFuture>
 
 #include "covermanager.h"
 #include "stringhash.h"
@@ -42,10 +43,9 @@ class QAction;
 class WebImageFetcher;
 class PlaylistItem;
 class PlaylistCollection;
-class PlaylistToolTip;
 class CollectionListItem;
 
-typedef QList<PlaylistItem *> PlaylistItemList;
+typedef QVector<PlaylistItem *> PlaylistItemList;
 
 class Playlist : public QTreeWidget, public PlaylistInterface
 {
@@ -80,7 +80,7 @@ public:
     virtual QString name() const;
     virtual FileHandle currentFile() const;
     virtual int count() const { return model()->rowCount(); }
-    virtual int time() const;
+    virtual int time() const { return m_time; }
     virtual void playNext();
     virtual void playPrevious();
     virtual void stop();
@@ -189,7 +189,7 @@ public:
      * items since it has the overhead of checking to see if the file is a playlist
      * or directory first.
      */
-    virtual void addFiles(const QStringList &files, PlaylistItem *after = 0);
+    virtual void addFiles(const QStringList &files, PlaylistItem *after = nullptr);
 
     /**
      * Returns the file name associated with this playlist (an m3u file) or
@@ -349,7 +349,6 @@ public slots:
      * @see clearItems()
      */
     virtual void clear();
-    virtual void selectAll() { QTreeView::selectAll(); }
 
     /**
      * Refreshes the tags of the selection from disk, or all of the files in the
@@ -466,8 +465,8 @@ protected:
      * As a template this allows us to use the same code to initialize the items
      * in subclasses. ItemType should be a PlaylistItem subclass.
      */
-    template <class ItemType, class SiblingType>
-    void createItems(const QList<SiblingType *> &siblings, ItemType *after = nullptr);
+    template <template <typename> class Container, class ItemType, class SiblingType>
+    void createItems(const Container<SiblingType *> &siblings, ItemType *after = nullptr);
 
 protected slots:
     void slotPopulateBackMenu() const;
@@ -493,7 +492,15 @@ signals:
 
     void signalPlaylistItemsDropped(Playlist *p);
 
+    void signalMoveFocusAway();
+
 private:
+    // Common constructor routines, inherited by other constructors
+    Playlist(
+            bool delaySetup, const QString &name,
+            PlaylistCollection *collection, const QString &iconName,
+            int extraCols);
+
     void setup();
 
     /**
@@ -538,10 +545,10 @@ private:
      */
     void calculateColumnWeights();
 
-    void addFile(const QString &file, FileHandleList &files, bool importPlaylists,
-                 PlaylistItem **after);
-    void addFileHelper(FileHandleList &files, PlaylistItem **after,
-                       bool ignoreTimer = false);
+    void addPlaylistFile(const QString &m3uFile);
+    QFuture<void> addFilesFromDirectory(const QString &dirPath);
+    QFuture<void> addUntypedFile(const QString &file, PlaylistItem *after = nullptr);
+    void cleanupAfterAllFileLoadsCompleted();
 
     void redisplaySearch() { setSearch(m_search); }
 
@@ -643,86 +650,61 @@ private slots:
     void slotInlineCompletionModeChanged(KCompletion::CompletionMode mode);
 
     void slotPlayCurrent();
+    void slotUpdateTime();
 
 private:
     friend class PlaylistItem;
 
-    PlaylistCollection *m_collection;
-
+    PlaylistCollection *m_collection = nullptr;
     StringHash m_members;
 
-    WebImageFetcher *m_fetcher;
+    // This is only defined if the playlist name is something other than the
+    // file name.
+    QString m_playlistName;
+    QString m_fileName;
 
-    QAction *m_rmbEdit;
-
-    bool m_allowDuplicates;
-    bool m_applySharedSettings;
-    bool m_columnWidthModeChanged;
-
-    QList<int> m_weightDirty;
-    bool m_disableColumnWidthUpdates;
-
-    mutable int m_time;
-    mutable PlaylistItemList m_addTime;
-    mutable PlaylistItemList m_subtractTime;
+    int  m_time            = 0;
+    bool m_allowDuplicates = true;
 
     /**
      * The average minimum widths of columns to be used in balancing calculations.
      */
+    QStringList m_columns;
     QVector<int> m_columnWeights;
     QVector<int> m_columnFixedWidths;
-    bool m_widthsDirty;
+    QVector<int> m_weightDirty;
+    KActionMenu *m_columnVisibleAction = nullptr;
+    bool m_columnWidthModeChanged      = false;
+    bool m_disableColumnWidthUpdates   = true;
+    bool m_widthsDirty                 = true;
+    bool m_applySharedSettings         = true;
 
-    static PlaylistItemList m_history;
     PlaylistSearch m_search;
+    bool m_searchEnabled = true;
 
-    bool m_searchEnabled;
+    int  m_itemsLoading = 0; /// Count of pending file loads outstanding
+    bool m_blockDataChanged = false;
 
-    /**
-     * Used to store the text for inline editing before it is changed so that
-     * we can know if something actually changed and as such if we need to save
-     * the tag.
-     */
-    QString m_editText;
-
-    /**
-     * This is only defined if the playlist name is something other than the
-     * file name.
-     */
-    QString m_playlistName;
-    QString m_fileName;
-
-    QStringList m_columns;
-    QMenu *m_rmbMenu;
-    QMenu *m_headerMenu;
-    KActionMenu *m_columnVisibleAction;
-    PlaylistToolTip *m_toolTip;
+    QAction *m_rmbEdit  = nullptr;
+    QMenu *m_rmbMenu    = nullptr;
+    QMenu *m_headerMenu = nullptr;
+    WebImageFetcher *m_fetcher = nullptr;
 
     /**
      * This is used to indicate if the list of visible items has changed (via a
      * call to setVisibleItems()) while random play is playing.
      */
     static bool m_visibleChanged;
+    static PlaylistItemList m_history;
     static bool m_shuttingDown;
     static int m_leftColumn;
     static QVector<PlaylistItem *> m_backMenuItems;
-
-    bool m_blockDataChanged;
 };
 
-typedef QList<Playlist *> PlaylistList;
+typedef QVector<Playlist *> PlaylistList;
 
 bool processEvents();
 
-class FocusUpEvent : public QEvent
-{
-public:
-    FocusUpEvent() : QEvent(id) {}
-    Type type() const { return id; }
-
-    static const Type id = static_cast<Type>(QEvent::User + 1);
-};
-
 QDataStream &operator<<(QDataStream &s, const Playlist &p);
 QDataStream &operator>>(QDataStream &s, Playlist &p);
 
@@ -756,8 +738,8 @@ ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
     return after;
 }
 
-template <class ItemType, class SiblingType>
-void Playlist::createItems(const QList<SiblingType *> &siblings, ItemType *after)
+template <template <typename> class Container, class ItemType, class SiblingType>
+void Playlist::createItems(const Container<SiblingType *> &siblings, ItemType *after)
 {
     if(siblings.isEmpty())
         return;
